/*
   Copyright (c) 2012 deeptought 
   
   Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
   and associated documentation files (the "Software"), to deal in the Software without restriction, 
   including without limitation the rights to use, copy, modify, merge, publish, distribute, 
   sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
   furnished to do so, subject to the following conditions: 
   
   The above copyright notice and this permission notice shall be included in all copies or 
   substantial portions of the Software. 
   
   The Software shall be used for Good, not Evil. 
   
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
   BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
 */
   
package de.uni.kassel.se;

import org.sdmlib.utils.PropertyChangeInterface;
import java.beans.PropertyChangeSupport;
import de.uni.kassel.se.creators.PlayerSet;
import java.util.LinkedHashSet;
import org.sdmlib.serialization.json.JsonIdMap;
import de.uni.kassel.se.creators.PitSet;

public class Mancala implements PropertyChangeInterface
{

   
   //==========================================================================
   
   public Object get(String attrName)
   {
      if (PROPERTY_PLAYERS.equalsIgnoreCase(attrName))
      {
         return getPlayers();
      }

      if (PROPERTY_PITS.equalsIgnoreCase(attrName))
      {
         return getPits();
      }

      return null;
   }

   
   //==========================================================================
   
   public boolean set(String attrName, Object value)
   {
      if (PROPERTY_PLAYERS.equalsIgnoreCase(attrName))
      {
         addToPlayers((Player) value);
         return true;
      }
      
      if ((PROPERTY_PLAYERS + JsonIdMap.REMOVE).equalsIgnoreCase(attrName))
      {
         removeFromPlayers((Player) value);
         return true;
      }

      if (PROPERTY_PITS.equalsIgnoreCase(attrName))
      {
         addToPits((Pit) value);
         return true;
      }
      
      if ((PROPERTY_PITS + JsonIdMap.REMOVE).equalsIgnoreCase(attrName))
      {
         removeFromPits((Pit) value);
         return true;
      }

      return false;
   }

   
   //==========================================================================
   
   protected PropertyChangeSupport listeners = new PropertyChangeSupport(this);
   
   public PropertyChangeSupport getPropertyChangeSupport()
   {
      return listeners;
   }

   
   //==========================================================================
   
   public void removeYou()
   {
      removeAllFromPlayers();
      removeAllFromPits();
      getPropertyChangeSupport().firePropertyChange("REMOVE_YOU", this, null);
   }

   
   //==========================================================================
   
   public void init( String p0, String p1 )
   {
      
   }

   
   /********************************************************************
    * <pre>
    *              one                       many
    * Mancala ----------------------------------- Player
    *              mancala                   players
    * </pre>
    */
   
   public static final String PROPERTY_PLAYERS = "players";
   
   private PlayerSet players = null;
   
   public PlayerSet getPlayers()
   {
      if (this.players == null)
      {
         return Player.EMPTY_SET;
      }
   
      return this.players;
   }
   
   public boolean addToPlayers(Player value)
   {
      boolean changed = false;
      
      if (value != null)
      {
         if (this.players == null)
         {
            this.players = new PlayerSet();
         }
         
         changed = this.players.add (value);
         
         if (changed)
         {
            value.withMancala(this);
            getPropertyChangeSupport().firePropertyChange(PROPERTY_PLAYERS, null, value);
         }
      }
         
      return changed;   
   }
   
   public boolean removeFromPlayers(Player value)
   {
      boolean changed = false;
      
      if ((this.players != null) && (value != null))
      {
         changed = this.players.remove (value);
         
         if (changed)
         {
            value.setMancala(null);
            getPropertyChangeSupport().firePropertyChange(PROPERTY_PLAYERS, value, null);
         }
      }
         
      return changed;   
   }
   
   public Mancala withPlayers(Player value)
   {
      addToPlayers(value);
      return this;
   } 
   
   public Mancala withoutPlayers(Player value)
   {
      removeFromPlayers(value);
      return this;
   } 
   
   public void removeAllFromPlayers()
   {
      LinkedHashSet<Player> tmpSet = new LinkedHashSet<Player>(this.getPlayers());
   
      for (Player value : tmpSet)
      {
         this.removeFromPlayers(value);
      }
   }
   
   public Player createPlayers()
   {
      Player value = new Player();
      withPlayers(value);
      return value;
   } 

   
   /********************************************************************
    * <pre>
    *              one                       many
    * Mancala ----------------------------------- Pit
    *              mancala                   pits
    * </pre>
    */
   
   public static final String PROPERTY_PITS = "pits";
   
   private PitSet pits = null;
   
   public PitSet getPits()
   {
      if (this.pits == null)
      {
         return Pit.EMPTY_SET;
      }
   
      return this.pits;
   }
   
   public boolean addToPits(Pit value)
   {
      boolean changed = false;
      
      if (value != null)
      {
         if (this.pits == null)
         {
            this.pits = new PitSet();
         }
         
         changed = this.pits.add (value);
         
         if (changed)
         {
            value.withMancala(this);
            getPropertyChangeSupport().firePropertyChange(PROPERTY_PITS, null, value);
         }
      }
         
      return changed;   
   }
   
   public boolean removeFromPits(Pit value)
   {
      boolean changed = false;
      
      if ((this.pits != null) && (value != null))
      {
         changed = this.pits.remove (value);
         
         if (changed)
         {
            value.setMancala(null);
            getPropertyChangeSupport().firePropertyChange(PROPERTY_PITS, value, null);
         }
      }
         
      return changed;   
   }
   
   public Mancala withPits(Pit value)
   {
      addToPits(value);
      return this;
   } 
   
   public Mancala withoutPits(Pit value)
   {
      removeFromPits(value);
      return this;
   } 
   
   public void removeAllFromPits()
   {
      LinkedHashSet<Pit> tmpSet = new LinkedHashSet<Pit>(this.getPits());
   
      for (Pit value : tmpSet)
      {
         this.removeFromPits(value);
      }
   }
   
   public Pit createPits()
   {
      Pit value = new Pit();
      withPits(value);
      return value;
   } 
}

