/*
   Copyright (c) 2012 deeptought 
   
   Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
   and associated documentation files (the "Software"), to deal in the Software without restriction, 
   including without limitation the rights to use, copy, modify, merge, publish, distribute, 
   sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
   furnished to do so, subject to the following conditions: 
   
   The above copyright notice and this permission notice shall be included in all copies or 
   substantial portions of the Software. 
   
   The Software shall be used for Good, not Evil. 
   
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
   BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
 */
   
package de.uni.kassel.se;

import org.sdmlib.utils.PropertyChangeInterface;
import java.beans.PropertyChangeSupport;
import de.uni.kassel.se.creators.PitSet;
import de.uni.kassel.se.creators.StoneSet;
import java.util.LinkedHashSet;
import org.sdmlib.serialization.json.JsonIdMap;

public class Pit implements PropertyChangeInterface
{

   
   //==========================================================================
   
   public Object get(String attrName)
   {
      if (PROPERTY_NR.equalsIgnoreCase(attrName))
      {
         return getNr();
      }

      if (PROPERTY_MANCALA.equalsIgnoreCase(attrName))
      {
         return getMancala();
      }

      if (PROPERTY_PLAYER.equalsIgnoreCase(attrName))
      {
         return getPlayer();
      }

      if (PROPERTY_NEXT.equalsIgnoreCase(attrName))
      {
         return getNext();
      }

      if (PROPERTY_PREVIOUS.equalsIgnoreCase(attrName))
      {
         return getPrevious();
      }

      if (PROPERTY_STONES.equalsIgnoreCase(attrName))
      {
         return getStones();
      }

      return null;
   }

   
   //==========================================================================
   
   public boolean set(String attrName, Object value)
   {
      if (PROPERTY_NR.equalsIgnoreCase(attrName))
      {
         setNr(Integer.parseInt(value.toString()));
         return true;
      }

      if (PROPERTY_MANCALA.equalsIgnoreCase(attrName))
      {
         setMancala((Mancala) value);
         return true;
      }

      if (PROPERTY_PLAYER.equalsIgnoreCase(attrName))
      {
         setPlayer((Player) value);
         return true;
      }

      if (PROPERTY_NEXT.equalsIgnoreCase(attrName))
      {
         setNext((Pit) value);
         return true;
      }

      if (PROPERTY_PREVIOUS.equalsIgnoreCase(attrName))
      {
         setPrevious((Pit) value);
         return true;
      }

      if (PROPERTY_STONES.equalsIgnoreCase(attrName))
      {
         addToStones((Stone) value);
         return true;
      }
      
      if ((PROPERTY_STONES + JsonIdMap.REMOVE).equalsIgnoreCase(attrName))
      {
         removeFromStones((Stone) value);
         return true;
      }

      return false;
   }

   
   //==========================================================================
   
   protected PropertyChangeSupport listeners = new PropertyChangeSupport(this);
   
   public PropertyChangeSupport getPropertyChangeSupport()
   {
      return listeners;
   }

   
   //==========================================================================
   
   public void removeYou()
   {
      setMancala(null);
      setPlayer(null);
      setNext(null);
      setPrevious(null);
      removeAllFromStones();
      getPropertyChangeSupport().firePropertyChange("REMOVE_YOU", this, null);
   }

   
   //==========================================================================
   
   public void moveStones(  )
   {
      
   }

   
   //==========================================================================
   
   public static final String PROPERTY_NR = "nr";
   
   private int nr;

   public int getNr()
   {
      return this.nr;
   }
   
   public void setNr(int value)
   {
      if (this.nr != value)
      {
         int oldValue = this.nr;
         this.nr = value;
         getPropertyChangeSupport().firePropertyChange(PROPERTY_NR, oldValue, value);
      }
   }
   
   public Pit withNr(int value)
   {
      setNr(value);
      return this;
   } 

   
   public static final PitSet EMPTY_SET = new PitSet();

   
   /********************************************************************
    * <pre>
    *              many                       one
    * Pit ----------------------------------- Mancala
    *              pits                   mancala
    * </pre>
    */
   
   public static final String PROPERTY_MANCALA = "mancala";
   
   private Mancala mancala = null;
   
   public Mancala getMancala()
   {
      return this.mancala;
   }
   
   public boolean setMancala(Mancala value)
   {
      boolean changed = false;
      
      if (this.mancala != value)
      {
         Mancala oldValue = this.mancala;
         
         if (this.mancala != null)
         {
            this.mancala = null;
            oldValue.withoutPits(this);
         }
         
         this.mancala = value;
         
         if (value != null)
         {
            value.withPits(this);
         }
         
         getPropertyChangeSupport().firePropertyChange(PROPERTY_MANCALA, oldValue, value);
         changed = true;
      }
      
      return changed;
   }
   
   public Pit withMancala(Mancala value)
   {
      setMancala(value);
      return this;
   } 
   
   public Mancala createMancala()
   {
      Mancala value = new Mancala();
      withMancala(value);
      return value;
   } 

   
   /********************************************************************
    * <pre>
    *              many                       one
    * Pit ----------------------------------- Player
    *              pits                   player
    * </pre>
    */
   
   public static final String PROPERTY_PLAYER = "player";
   
   private Player player = null;
   
   public Player getPlayer()
   {
      return this.player;
   }
   
   public boolean setPlayer(Player value)
   {
      boolean changed = false;
      
      if (this.player != value)
      {
         Player oldValue = this.player;
         
         if (this.player != null)
         {
            this.player = null;
            oldValue.withoutPits(this);
         }
         
         this.player = value;
         
         if (value != null)
         {
            value.withPits(this);
         }
         
         getPropertyChangeSupport().firePropertyChange(PROPERTY_PLAYER, oldValue, value);
         changed = true;
      }
      
      return changed;
   }
   
   public Pit withPlayer(Player value)
   {
      setPlayer(value);
      return this;
   } 
   
   public Player createPlayer()
   {
      Player value = new Player();
      withPlayer(value);
      return value;
   } 

   
   /********************************************************************
    * <pre>
    *              one                       one
    * Pit ----------------------------------- Pit
    *              previous                   next
    * </pre>
    */
   
   public static final String PROPERTY_NEXT = "next";
   
   private Pit next = null;
   
   public Pit getNext()
   {
      return this.next;
   }
   
   public boolean setNext(Pit value)
   {
      boolean changed = false;
      
      if (this.next != value)
      {
         Pit oldValue = this.next;
         
         if (this.next != null)
         {
            this.next = null;
            oldValue.setPrevious(null);
         }
         
         this.next = value;
         
         if (value != null)
         {
            value.withPrevious(this);
         }
         
         getPropertyChangeSupport().firePropertyChange(PROPERTY_NEXT, oldValue, value);
         changed = true;
      }
      
      return changed;
   }
   
   public Pit withNext(Pit value)
   {
      setNext(value);
      return this;
   } 
   
   public Pit createNext()
   {
      Pit value = new Pit();
      withNext(value);
      return value;
   } 

   
   /********************************************************************
    * <pre>
    *              one                       one
    * Pit ----------------------------------- Pit
    *              next                   previous
    * </pre>
    */
   
   public static final String PROPERTY_PREVIOUS = "previous";
   
   private Pit previous = null;
   
   public Pit getPrevious()
   {
      return this.previous;
   }
   
   public boolean setPrevious(Pit value)
   {
      boolean changed = false;
      
      if (this.previous != value)
      {
         Pit oldValue = this.previous;
         
         if (this.previous != null)
         {
            this.previous = null;
            oldValue.setNext(null);
         }
         
         this.previous = value;
         
         if (value != null)
         {
            value.withNext(this);
         }
         
         getPropertyChangeSupport().firePropertyChange(PROPERTY_PREVIOUS, oldValue, value);
         changed = true;
      }
      
      return changed;
   }
   
   public Pit withPrevious(Pit value)
   {
      setPrevious(value);
      return this;
   } 
   
   public Pit createPrevious()
   {
      Pit value = new Pit();
      withPrevious(value);
      return value;
   } 

   
   /********************************************************************
    * <pre>
    *              one                       many
    * Pit ----------------------------------- Stone
    *              pit                   stones
    * </pre>
    */
   
   public static final String PROPERTY_STONES = "stones";
   
   private StoneSet stones = null;
   
   public StoneSet getStones()
   {
      if (this.stones == null)
      {
         return Stone.EMPTY_SET;
      }
   
      return this.stones;
   }
   
   public boolean addToStones(Stone value)
   {
      boolean changed = false;
      
      if (value != null)
      {
         if (this.stones == null)
         {
            this.stones = new StoneSet();
         }
         
         changed = this.stones.add (value);
         
         if (changed)
         {
            value.withPit(this);
            getPropertyChangeSupport().firePropertyChange(PROPERTY_STONES, null, value);
         }
      }
         
      return changed;   
   }
   
   public boolean removeFromStones(Stone value)
   {
      boolean changed = false;
      
      if ((this.stones != null) && (value != null))
      {
         changed = this.stones.remove (value);
         
         if (changed)
         {
            value.setPit(null);
            getPropertyChangeSupport().firePropertyChange(PROPERTY_STONES, value, null);
         }
      }
         
      return changed;   
   }
   
   public Pit withStones(Stone value)
   {
      addToStones(value);
      return this;
   } 
   
   public Pit withoutStones(Stone value)
   {
      removeFromStones(value);
      return this;
   } 
   
   public void removeAllFromStones()
   {
      LinkedHashSet<Stone> tmpSet = new LinkedHashSet<Stone>(this.getStones());
   
      for (Stone value : tmpSet)
      {
         this.removeFromStones(value);
      }
   }
   
   public Stone createStones()
   {
      Stone value = new Stone();
      withStones(value);
      return value;
   } 
}

