/*
   Copyright (c) 2012 deeptought 
   
   Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
   and associated documentation files (the "Software"), to deal in the Software without restriction, 
   including without limitation the rights to use, copy, modify, merge, publish, distribute, 
   sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
   furnished to do so, subject to the following conditions: 
   
   The above copyright notice and this permission notice shall be included in all copies or 
   substantial portions of the Software. 
   
   The Software shall be used for Good, not Evil. 
   
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
   BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
 */
   
package de.uni.kassel.se;

import org.sdmlib.utils.PropertyChangeInterface;
import java.beans.PropertyChangeSupport;
import org.sdmlib.utils.StrUtil;
import de.uni.kassel.se.creators.PlayerSet;
import de.uni.kassel.se.creators.PitSet;
import java.util.LinkedHashSet;
import org.sdmlib.serialization.json.JsonIdMap;

public class Player implements PropertyChangeInterface
{

   
   //==========================================================================
   
   public Object get(String attrName)
   {
      if (PROPERTY_NAME.equalsIgnoreCase(attrName))
      {
         return getName();
      }

      if (PROPERTY_MANCALA.equalsIgnoreCase(attrName))
      {
         return getMancala();
      }

      if (PROPERTY_PITS.equalsIgnoreCase(attrName))
      {
         return getPits();
      }

      return null;
   }

   
   //==========================================================================
   
   public boolean set(String attrName, Object value)
   {
      if (PROPERTY_NAME.equalsIgnoreCase(attrName))
      {
         setName((String) value);
         return true;
      }

      if (PROPERTY_MANCALA.equalsIgnoreCase(attrName))
      {
         setMancala((Mancala) value);
         return true;
      }

      if (PROPERTY_PITS.equalsIgnoreCase(attrName))
      {
         addToPits((Pit) value);
         return true;
      }
      
      if ((PROPERTY_PITS + JsonIdMap.REMOVE).equalsIgnoreCase(attrName))
      {
         removeFromPits((Pit) value);
         return true;
      }

      return false;
   }

   
   //==========================================================================
   
   protected PropertyChangeSupport listeners = new PropertyChangeSupport(this);
   
   public PropertyChangeSupport getPropertyChangeSupport()
   {
      return listeners;
   }

   
   //==========================================================================
   
   public void removeYou()
   {
      setMancala(null);
      removeAllFromPits();
      getPropertyChangeSupport().firePropertyChange("REMOVE_YOU", this, null);
   }

   
   //==========================================================================
   
   public static final String PROPERTY_NAME = "name";
   
   private String name;

   public String getName()
   {
      return this.name;
   }
   
   public void setName(String value)
   {
      if ( ! StrUtil.stringEquals(this.name, value))
      {
         String oldValue = this.name;
         this.name = value;
         getPropertyChangeSupport().firePropertyChange(PROPERTY_NAME, oldValue, value);
      }
   }
   
   public Player withName(String value)
   {
      setName(value);
      return this;
   } 

   
   public static final PlayerSet EMPTY_SET = new PlayerSet();

   
   /********************************************************************
    * <pre>
    *              many                       one
    * Player ----------------------------------- Mancala
    *              players                   mancala
    * </pre>
    */
   
   public static final String PROPERTY_MANCALA = "mancala";
   
   private Mancala mancala = null;
   
   public Mancala getMancala()
   {
      return this.mancala;
   }
   
   public boolean setMancala(Mancala value)
   {
      boolean changed = false;
      
      if (this.mancala != value)
      {
         Mancala oldValue = this.mancala;
         
         if (this.mancala != null)
         {
            this.mancala = null;
            oldValue.withoutPlayers(this);
         }
         
         this.mancala = value;
         
         if (value != null)
         {
            value.withPlayers(this);
         }
         
         getPropertyChangeSupport().firePropertyChange(PROPERTY_MANCALA, oldValue, value);
         changed = true;
      }
      
      return changed;
   }
   
   public Player withMancala(Mancala value)
   {
      setMancala(value);
      return this;
   } 
   
   public Mancala createMancala()
   {
      Mancala value = new Mancala();
      withMancala(value);
      return value;
   } 

   
   /********************************************************************
    * <pre>
    *              one                       many
    * Player ----------------------------------- Pit
    *              player                   pits
    * </pre>
    */
   
   public static final String PROPERTY_PITS = "pits";
   
   private PitSet pits = null;
   
   public PitSet getPits()
   {
      if (this.pits == null)
      {
         return Pit.EMPTY_SET;
      }
   
      return this.pits;
   }
   
   public boolean addToPits(Pit value)
   {
      boolean changed = false;
      
      if (value != null)
      {
         if (this.pits == null)
         {
            this.pits = new PitSet();
         }
         
         changed = this.pits.add (value);
         
         if (changed)
         {
            value.withPlayer(this);
            getPropertyChangeSupport().firePropertyChange(PROPERTY_PITS, null, value);
         }
      }
         
      return changed;   
   }
   
   public boolean removeFromPits(Pit value)
   {
      boolean changed = false;
      
      if ((this.pits != null) && (value != null))
      {
         changed = this.pits.remove (value);
         
         if (changed)
         {
            value.setPlayer(null);
            getPropertyChangeSupport().firePropertyChange(PROPERTY_PITS, value, null);
         }
      }
         
      return changed;   
   }
   
   public Player withPits(Pit value)
   {
      addToPits(value);
      return this;
   } 
   
   public Player withoutPits(Pit value)
   {
      removeFromPits(value);
      return this;
   } 
   
   public void removeAllFromPits()
   {
      LinkedHashSet<Pit> tmpSet = new LinkedHashSet<Pit>(this.getPits());
   
      for (Pit value : tmpSet)
      {
         this.removeFromPits(value);
      }
   }
   
   public Pit createPits()
   {
      Pit value = new Pit();
      withPits(value);
      return value;
   } 
}

