package de.uni.kassel.se.creators;

import org.sdmlib.models.pattern.PatternObject;
import de.uni.kassel.se.Pit;
import de.uni.kassel.se.creators.PitSet;
import org.sdmlib.models.pattern.AttributeConstraint;
import org.sdmlib.models.pattern.PatternLink;
import de.uni.kassel.se.creators.MancalaPO;
import org.sdmlib.models.pattern.LinkConstraint;
import de.uni.kassel.se.creators.PitPO;
import de.uni.kassel.se.Mancala;
import de.uni.kassel.se.creators.PlayerPO;
import de.uni.kassel.se.Player;
import de.uni.kassel.se.creators.StonePO;
import de.uni.kassel.se.Stone;
import de.uni.kassel.se.creators.StoneSet;

public class PitPO extends PatternObject<PitPO, Pit>
{
   public PitSet allMatches()
   {
      this.setDoAllMatches(true);
      
      PitSet matches = new PitSet();

      while (this.getPattern().getHasMatch())
      {
         matches.add((Pit) this.getCurrentMatch());
         
         this.getPattern().findMatch();
      }
      
      return matches;
   }
   
   
   //==========================================================================
   
   public void moveStones()
   {
      if (this.getPattern().getHasMatch())
      {
          ((Pit) getCurrentMatch()).moveStones();
      }
   }

   public PitPO hasNr(int value)
   {
      AttributeConstraint constr = (AttributeConstraint) new AttributeConstraint()
      .withAttrName(Pit.PROPERTY_NR)
      .withTgtValue(value)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier())
      .withPattern(this.getPattern());
      
      this.getPattern().findMatch();
      
      return this;
   }
   
   public int getNr()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Pit) getCurrentMatch()).getNr();
      }
      return 0;
   }
   
   public PitPO withNr(int value)
   {
      if (this.getPattern().getHasMatch())
      {
         ((Pit) getCurrentMatch()).setNr(value);
      }
      return this;
   }
   
   public MancalaPO hasMancala()
   {
      MancalaPO result = new MancalaPO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Pit.PROPERTY_MANCALA, result);
      
      return result;
   }
   
   public PitPO hasMancala(MancalaPO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Pit.PROPERTY_MANCALA)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }
   
   public Mancala getMancala()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Pit) this.getCurrentMatch()).getMancala();
      }
      return null;
   }
   
   public PlayerPO hasPlayer()
   {
      PlayerPO result = new PlayerPO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Pit.PROPERTY_PLAYER, result);
      
      return result;
   }
   
   public PitPO hasPlayer(PlayerPO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Pit.PROPERTY_PLAYER)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }
   
   public Player getPlayer()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Pit) this.getCurrentMatch()).getPlayer();
      }
      return null;
   }
   
   public PitPO hasNext()
   {
      PitPO result = new PitPO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Pit.PROPERTY_NEXT, result);
      
      return result;
   }
   
   public PitPO hasNext(PitPO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Pit.PROPERTY_NEXT)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }
   
   public Pit getNext()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Pit) this.getCurrentMatch()).getNext();
      }
      return null;
   }
   
   public PitPO hasPrevious()
   {
      PitPO result = new PitPO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Pit.PROPERTY_PREVIOUS, result);
      
      return result;
   }
   
   public PitPO hasPrevious(PitPO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Pit.PROPERTY_PREVIOUS)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }
   
   public Pit getPrevious()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Pit) this.getCurrentMatch()).getPrevious();
      }
      return null;
   }
   
   public StonePO hasStones()
   {
      StonePO result = new StonePO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Pit.PROPERTY_STONES, result);
      
      return result;
   }
   
   public PitPO hasStones(StonePO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Pit.PROPERTY_STONES)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }
   
   public StoneSet getStones()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Pit) this.getCurrentMatch()).getStones();
      }
      return null;
   }
   
}

