package de.uni.kassel.se.creators;

import org.sdmlib.models.pattern.PatternObject;
import de.uni.kassel.se.Player;
import de.uni.kassel.se.creators.PlayerSet;
import org.sdmlib.models.pattern.AttributeConstraint;
import org.sdmlib.models.pattern.PatternLink;
import de.uni.kassel.se.creators.MancalaPO;
import org.sdmlib.models.pattern.LinkConstraint;
import de.uni.kassel.se.creators.PlayerPO;
import de.uni.kassel.se.Mancala;
import de.uni.kassel.se.creators.PitPO;
import de.uni.kassel.se.Pit;
import de.uni.kassel.se.creators.PitSet;

public class PlayerPO extends PatternObject<PlayerPO, Player>
{
   public PlayerSet allMatches()
   {
      this.setDoAllMatches(true);
      
      PlayerSet matches = new PlayerSet();

      while (this.getPattern().getHasMatch())
      {
         matches.add((Player) this.getCurrentMatch());
         
         this.getPattern().findMatch();
      }
      
      return matches;
   }
   
   public PlayerPO hasName(String value)
   {
      AttributeConstraint constr = (AttributeConstraint) new AttributeConstraint()
      .withAttrName(Player.PROPERTY_NAME)
      .withTgtValue(value)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier())
      .withPattern(this.getPattern());
      
      this.getPattern().findMatch();
      
      return this;
   }
   
   public String getName()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Player) getCurrentMatch()).getName();
      }
      return null;
   }
   
   public PlayerPO withName(String value)
   {
      if (this.getPattern().getHasMatch())
      {
         ((Player) getCurrentMatch()).setName(value);
      }
      return this;
   }
   
   public MancalaPO hasMancala()
   {
      MancalaPO result = new MancalaPO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Player.PROPERTY_MANCALA, result);
      
      return result;
   }
   
   public PlayerPO hasMancala(MancalaPO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Player.PROPERTY_MANCALA)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }
   
   public Mancala getMancala()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Player) this.getCurrentMatch()).getMancala();
      }
      return null;
   }
   
   public PitPO hasPits()
   {
      PitPO result = new PitPO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Player.PROPERTY_PITS, result);
      
      return result;
   }
   
   public PlayerPO hasPits(PitPO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Player.PROPERTY_PITS)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }
   
   public PitSet getPits()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Player) this.getCurrentMatch()).getPits();
      }
      return null;
   }
   
}

